# pylint: disable=line-too-long,useless-suppression,too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=useless-super-delegation

import datetime
from typing import Any, Mapping, Optional, TYPE_CHECKING, Union, overload

from .._utils.model_base import Model as _Model, rest_field

if TYPE_CHECKING:
    from .. import models as _models


class AccountSku(_Model):
    """SKU of the artifact signing account.

    :ivar name: Name of the SKU. Required. Known values are: "Basic" and "Premium".
    :vartype name: str or ~azure.mgmt.artifactsigning.models.SkuName
    """

    name: Union[str, "_models.SkuName"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Name of the SKU. Required. Known values are: \"Basic\" and \"Premium\"."""

    @overload
    def __init__(
        self,
        *,
        name: Union[str, "_models.SkuName"],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class AccountSkuPatch(_Model):
    """SKU of the artifact signing account.

    :ivar name: Name of the SKU. Known values are: "Basic" and "Premium".
    :vartype name: str or ~azure.mgmt.artifactsigning.models.SkuName
    """

    name: Optional[Union[str, "_models.SkuName"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Name of the SKU. Known values are: \"Basic\" and \"Premium\"."""

    @overload
    def __init__(
        self,
        *,
        name: Optional[Union[str, "_models.SkuName"]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class Certificate(_Model):
    """Properties of the certificate.

    :ivar serial_number: Serial number of the certificate.
    :vartype serial_number: str
    :ivar enhanced_key_usage: Enhanced key usage of the certificate.
    :vartype enhanced_key_usage: str
    :ivar subject_name: Subject name of the certificate.
    :vartype subject_name: str
    :ivar thumbprint: Thumbprint of the certificate.
    :vartype thumbprint: str
    :ivar created_date: Certificate created date.
    :vartype created_date: str
    :ivar expiry_date: Certificate expiry date.
    :vartype expiry_date: str
    :ivar status: Status of the certificate. Known values are: "Active", "Expired", and "Revoked".
    :vartype status: str or ~azure.mgmt.artifactsigning.models.CertificateStatus
    :ivar revocation: Revocations history of a certificate.
    :vartype revocation: ~azure.mgmt.artifactsigning.models.Revocation
    """

    serial_number: Optional[str] = rest_field(
        name="serialNumber", visibility=["read", "create", "update", "delete", "query"]
    )
    """Serial number of the certificate."""
    enhanced_key_usage: Optional[str] = rest_field(
        name="enhancedKeyUsage", visibility=["read", "create", "update", "delete", "query"]
    )
    """Enhanced key usage of the certificate."""
    subject_name: Optional[str] = rest_field(
        name="subjectName", visibility=["read", "create", "update", "delete", "query"]
    )
    """Subject name of the certificate."""
    thumbprint: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Thumbprint of the certificate."""
    created_date: Optional[str] = rest_field(
        name="createdDate", visibility=["read", "create", "update", "delete", "query"]
    )
    """Certificate created date."""
    expiry_date: Optional[str] = rest_field(
        name="expiryDate", visibility=["read", "create", "update", "delete", "query"]
    )
    """Certificate expiry date."""
    status: Optional[Union[str, "_models.CertificateStatus"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Status of the certificate. Known values are: \"Active\", \"Expired\", and \"Revoked\"."""
    revocation: Optional["_models.Revocation"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Revocations history of a certificate."""

    __flattened_items = ["requested_at", "effective_at", "reason", "remarks", "status", "failure_reason"]

    @overload
    def __init__(
        self,
        *,
        serial_number: Optional[str] = None,
        enhanced_key_usage: Optional[str] = None,
        subject_name: Optional[str] = None,
        thumbprint: Optional[str] = None,
        created_date: Optional[str] = None,
        expiry_date: Optional[str] = None,
        status: Optional[Union[str, "_models.CertificateStatus"]] = None,
        revocation: Optional["_models.Revocation"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        _flattened_input = {k: kwargs.pop(k) for k in kwargs.keys() & self.__flattened_items}
        super().__init__(*args, **kwargs)
        for k, v in _flattened_input.items():
            setattr(self, k, v)

    def __getattr__(self, name: str) -> Any:
        if name in self.__flattened_items:
            if self.revocation is None:
                return None
            return getattr(self.revocation, name)
        raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")

    def __setattr__(self, key: str, value: Any) -> None:
        if key in self.__flattened_items:
            if self.revocation is None:
                self.revocation = self._attr_to_rest_field["revocation"]._class_type()
            setattr(self.revocation, key, value)
        else:
            super().__setattr__(key, value)


class Resource(_Model):
    """Resource.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.mgmt.artifactsigning.models.SystemData
    """

    id: Optional[str] = rest_field(visibility=["read"])
    """Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}."""
    name: Optional[str] = rest_field(visibility=["read"])
    """The name of the resource."""
    type: Optional[str] = rest_field(visibility=["read"])
    """The type of the resource. E.g. \"Microsoft.Compute/virtualMachines\" or
     \"Microsoft.Storage/storageAccounts\"."""
    system_data: Optional["_models.SystemData"] = rest_field(name="systemData", visibility=["read"])
    """Azure Resource Manager metadata containing createdBy and modifiedBy information."""


class ProxyResource(Resource):
    """Proxy Resource.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.mgmt.artifactsigning.models.SystemData
    """


class CertificateProfile(ProxyResource):
    """Certificate profile resource.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.mgmt.artifactsigning.models.SystemData
    :ivar properties: The resource-specific properties for this resource.
    :vartype properties: ~azure.mgmt.artifactsigning.models.CertificateProfileProperties
    """

    properties: Optional["_models.CertificateProfileProperties"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The resource-specific properties for this resource."""

    __flattened_items = [
        "profile_type",
        "include_street_address",
        "include_city",
        "include_state",
        "include_country",
        "include_postal_code",
        "identity_validation_id",
        "provisioning_state",
        "status",
        "certificates",
    ]

    @overload
    def __init__(
        self,
        *,
        properties: Optional["_models.CertificateProfileProperties"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        _flattened_input = {k: kwargs.pop(k) for k in kwargs.keys() & self.__flattened_items}
        super().__init__(*args, **kwargs)
        for k, v in _flattened_input.items():
            setattr(self, k, v)

    def __getattr__(self, name: str) -> Any:
        if name in self.__flattened_items:
            if self.properties is None:
                return None
            return getattr(self.properties, name)
        raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")

    def __setattr__(self, key: str, value: Any) -> None:
        if key in self.__flattened_items:
            if self.properties is None:
                self.properties = self._attr_to_rest_field["properties"]._class_type()
            setattr(self.properties, key, value)
        else:
            super().__setattr__(key, value)


class CertificateProfileProperties(_Model):
    """Properties of the certificate profile.

    :ivar profile_type: Profile type of the certificate. Required. Known values are: "PublicTrust",
     "PrivateTrust", "PrivateTrustCIPolicy", "VBSEnclave", and "PublicTrustTest".
    :vartype profile_type: str or ~azure.mgmt.artifactsigning.models.ProfileType
    :ivar include_street_address: Whether to include STREET in the certificate subject name.
    :vartype include_street_address: bool
    :ivar include_city: Whether to include L in the certificate subject name. Applicable only for
     private trust, private trust ci profile types.
    :vartype include_city: bool
    :ivar include_state: Whether to include S in the certificate subject name. Applicable only for
     private trust, private trust ci profile types.
    :vartype include_state: bool
    :ivar include_country: Whether to include C in the certificate subject name. Applicable only
     for private trust, private trust ci profile types.
    :vartype include_country: bool
    :ivar include_postal_code: Whether to include PC in the certificate subject name.
    :vartype include_postal_code: bool
    :ivar identity_validation_id: Identity validation id used for the certificate subject name.
     Required.
    :vartype identity_validation_id: str
    :ivar provisioning_state: Status of the current operation on certificate profile. Known values
     are: "Succeeded", "Failed", "Canceled", "Updating", "Deleting", and "Accepted".
    :vartype provisioning_state: str or ~azure.mgmt.artifactsigning.models.ProvisioningState
    :ivar status: Status of the certificate profile. Known values are: "Active", "Disabled", and
     "Suspended".
    :vartype status: str or ~azure.mgmt.artifactsigning.models.CertificateProfileStatus
    :ivar certificates: List of renewed certificates.
    :vartype certificates: list[~azure.mgmt.artifactsigning.models.Certificate]
    """

    profile_type: Union[str, "_models.ProfileType"] = rest_field(
        name="profileType", visibility=["read", "create", "update", "delete", "query"]
    )
    """Profile type of the certificate. Required. Known values are: \"PublicTrust\", \"PrivateTrust\",
     \"PrivateTrustCIPolicy\", \"VBSEnclave\", and \"PublicTrustTest\"."""
    include_street_address: Optional[bool] = rest_field(
        name="includeStreetAddress", visibility=["read", "create", "update", "delete", "query"]
    )
    """Whether to include STREET in the certificate subject name."""
    include_city: Optional[bool] = rest_field(
        name="includeCity", visibility=["read", "create", "update", "delete", "query"]
    )
    """Whether to include L in the certificate subject name. Applicable only for private trust,
     private trust ci profile types."""
    include_state: Optional[bool] = rest_field(
        name="includeState", visibility=["read", "create", "update", "delete", "query"]
    )
    """Whether to include S in the certificate subject name. Applicable only for private trust,
     private trust ci profile types."""
    include_country: Optional[bool] = rest_field(
        name="includeCountry", visibility=["read", "create", "update", "delete", "query"]
    )
    """Whether to include C in the certificate subject name. Applicable only for private trust,
     private trust ci profile types."""
    include_postal_code: Optional[bool] = rest_field(
        name="includePostalCode", visibility=["read", "create", "update", "delete", "query"]
    )
    """Whether to include PC in the certificate subject name."""
    identity_validation_id: str = rest_field(
        name="identityValidationId", visibility=["read", "create", "update", "delete", "query"]
    )
    """Identity validation id used for the certificate subject name. Required."""
    provisioning_state: Optional[Union[str, "_models.ProvisioningState"]] = rest_field(
        name="provisioningState", visibility=["read"]
    )
    """Status of the current operation on certificate profile. Known values are: \"Succeeded\",
     \"Failed\", \"Canceled\", \"Updating\", \"Deleting\", and \"Accepted\"."""
    status: Optional[Union[str, "_models.CertificateProfileStatus"]] = rest_field(visibility=["read"])
    """Status of the certificate profile. Known values are: \"Active\", \"Disabled\", and
     \"Suspended\"."""
    certificates: Optional[list["_models.Certificate"]] = rest_field(visibility=["read"])
    """List of renewed certificates."""

    @overload
    def __init__(
        self,
        *,
        profile_type: Union[str, "_models.ProfileType"],
        identity_validation_id: str,
        include_street_address: Optional[bool] = None,
        include_city: Optional[bool] = None,
        include_state: Optional[bool] = None,
        include_country: Optional[bool] = None,
        include_postal_code: Optional[bool] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class CheckNameAvailability(_Model):
    """The parameters used to check the availability of the artifact signing account name.

    :ivar type: The type of the resource, "Microsoft.CodeSigning/codeSigningAccounts". Required.
    :vartype type: str
    :ivar name: Artifact signing account name. Required.
    :vartype name: str
    """

    type: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The type of the resource, \"Microsoft.CodeSigning/codeSigningAccounts\". Required."""
    name: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Artifact signing account name. Required."""

    @overload
    def __init__(
        self,
        *,
        type: str,
        name: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class CheckNameAvailabilityResult(_Model):
    """The CheckNameAvailability operation response.

    :ivar name_available: A boolean value that indicates whether the name is available for you to
     use. If true, the name is available. If false, the name has already been taken or is invalid
     and cannot be used.
    :vartype name_available: bool
    :ivar reason: The reason that an artifact signing account name could not be used. The Reason
     element is only returned if nameAvailable is false. Known values are: "AccountNameInvalid" and
     "AlreadyExists".
    :vartype reason: str or ~azure.mgmt.artifactsigning.models.NameUnavailabilityReason
    :ivar message: An error message explaining the Reason value in more detail.
    :vartype message: str
    """

    name_available: Optional[bool] = rest_field(name="nameAvailable", visibility=["read"])
    """A boolean value that indicates whether the name is available for you to use. If true, the name
     is available. If false, the name has already been taken or is invalid and cannot be used."""
    reason: Optional[Union[str, "_models.NameUnavailabilityReason"]] = rest_field(visibility=["read"])
    """The reason that an artifact signing account name could not be used. The Reason element is only
     returned if nameAvailable is false. Known values are: \"AccountNameInvalid\" and
     \"AlreadyExists\"."""
    message: Optional[str] = rest_field(visibility=["read"])
    """An error message explaining the Reason value in more detail."""


class TrackedResource(Resource):
    """Tracked Resource.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.mgmt.artifactsigning.models.SystemData
    :ivar tags: Resource tags.
    :vartype tags: dict[str, str]
    :ivar location: The geo-location where the resource lives. Required.
    :vartype location: str
    """

    tags: Optional[dict[str, str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Resource tags."""
    location: str = rest_field(visibility=["read", "create"])
    """The geo-location where the resource lives. Required."""

    @overload
    def __init__(
        self,
        *,
        location: str,
        tags: Optional[dict[str, str]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class CodeSigningAccount(TrackedResource):
    """Artifact signing account resource.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.mgmt.artifactsigning.models.SystemData
    :ivar tags: Resource tags.
    :vartype tags: dict[str, str]
    :ivar location: The geo-location where the resource lives. Required.
    :vartype location: str
    :ivar properties: The resource-specific properties for this resource.
    :vartype properties: ~azure.mgmt.artifactsigning.models.CodeSigningAccountProperties
    """

    properties: Optional["_models.CodeSigningAccountProperties"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The resource-specific properties for this resource."""

    __flattened_items = ["account_uri", "sku", "provisioning_state"]

    @overload
    def __init__(
        self,
        *,
        location: str,
        tags: Optional[dict[str, str]] = None,
        properties: Optional["_models.CodeSigningAccountProperties"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        _flattened_input = {k: kwargs.pop(k) for k in kwargs.keys() & self.__flattened_items}
        super().__init__(*args, **kwargs)
        for k, v in _flattened_input.items():
            setattr(self, k, v)

    def __getattr__(self, name: str) -> Any:
        if name in self.__flattened_items:
            if self.properties is None:
                return None
            return getattr(self.properties, name)
        raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")

    def __setattr__(self, key: str, value: Any) -> None:
        if key in self.__flattened_items:
            if self.properties is None:
                self.properties = self._attr_to_rest_field["properties"]._class_type()
            setattr(self.properties, key, value)
        else:
            super().__setattr__(key, value)


class CodeSigningAccountPatch(_Model):
    """Parameters for creating or updating an artifact signing account.

    :ivar tags: Resource tags.
    :vartype tags: dict[str, str]
    :ivar properties: Properties of the artifact signing account.
    :vartype properties: ~azure.mgmt.artifactsigning.models.CodeSigningAccountPatchProperties
    """

    tags: Optional[dict[str, str]] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Resource tags."""
    properties: Optional["_models.CodeSigningAccountPatchProperties"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Properties of the artifact signing account."""

    __flattened_items = ["sku"]

    @overload
    def __init__(
        self,
        *,
        tags: Optional[dict[str, str]] = None,
        properties: Optional["_models.CodeSigningAccountPatchProperties"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        _flattened_input = {k: kwargs.pop(k) for k in kwargs.keys() & self.__flattened_items}
        super().__init__(*args, **kwargs)
        for k, v in _flattened_input.items():
            setattr(self, k, v)

    def __getattr__(self, name: str) -> Any:
        if name in self.__flattened_items:
            if self.properties is None:
                return None
            return getattr(self.properties, name)
        raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")

    def __setattr__(self, key: str, value: Any) -> None:
        if key in self.__flattened_items:
            if self.properties is None:
                self.properties = self._attr_to_rest_field["properties"]._class_type()
            setattr(self.properties, key, value)
        else:
            super().__setattr__(key, value)


class CodeSigningAccountPatchProperties(_Model):
    """Properties of the artifact signing account.

    :ivar sku: SKU of the artifact signing account.
    :vartype sku: ~azure.mgmt.artifactsigning.models.AccountSkuPatch
    """

    sku: Optional["_models.AccountSkuPatch"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """SKU of the artifact signing account."""

    @overload
    def __init__(
        self,
        *,
        sku: Optional["_models.AccountSkuPatch"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class CodeSigningAccountProperties(_Model):
    """Properties of the artifact signing account.

    :ivar account_uri: The URI of the artifact signing account which is used during signing files.
    :vartype account_uri: str
    :ivar sku: SKU of the artifact signing account.
    :vartype sku: ~azure.mgmt.artifactsigning.models.AccountSku
    :ivar provisioning_state: Status of the current operation on artifact signing account. Known
     values are: "Succeeded", "Failed", "Canceled", "Updating", "Deleting", and "Accepted".
    :vartype provisioning_state: str or ~azure.mgmt.artifactsigning.models.ProvisioningState
    """

    account_uri: Optional[str] = rest_field(name="accountUri", visibility=["read"])
    """The URI of the artifact signing account which is used during signing files."""
    sku: Optional["_models.AccountSku"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """SKU of the artifact signing account."""
    provisioning_state: Optional[Union[str, "_models.ProvisioningState"]] = rest_field(
        name="provisioningState", visibility=["read"]
    )
    """Status of the current operation on artifact signing account. Known values are: \"Succeeded\",
     \"Failed\", \"Canceled\", \"Updating\", \"Deleting\", and \"Accepted\"."""

    @overload
    def __init__(
        self,
        *,
        sku: Optional["_models.AccountSku"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ErrorAdditionalInfo(_Model):
    """The resource management error additional info.

    :ivar type: The additional info type.
    :vartype type: str
    :ivar info: The additional info.
    :vartype info: any
    """

    type: Optional[str] = rest_field(visibility=["read"])
    """The additional info type."""
    info: Optional[Any] = rest_field(visibility=["read"])
    """The additional info."""


class ErrorDetail(_Model):
    """The error detail.

    :ivar code: The error code.
    :vartype code: str
    :ivar message: The error message.
    :vartype message: str
    :ivar target: The error target.
    :vartype target: str
    :ivar details: The error details.
    :vartype details: list[~azure.mgmt.artifactsigning.models.ErrorDetail]
    :ivar additional_info: The error additional info.
    :vartype additional_info: list[~azure.mgmt.artifactsigning.models.ErrorAdditionalInfo]
    """

    code: Optional[str] = rest_field(visibility=["read"])
    """The error code."""
    message: Optional[str] = rest_field(visibility=["read"])
    """The error message."""
    target: Optional[str] = rest_field(visibility=["read"])
    """The error target."""
    details: Optional[list["_models.ErrorDetail"]] = rest_field(visibility=["read"])
    """The error details."""
    additional_info: Optional[list["_models.ErrorAdditionalInfo"]] = rest_field(
        name="additionalInfo", visibility=["read"]
    )
    """The error additional info."""


class ErrorResponse(_Model):
    """Error response.

    :ivar error: The error object.
    :vartype error: ~azure.mgmt.artifactsigning.models.ErrorDetail
    """

    error: Optional["_models.ErrorDetail"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The error object."""

    @overload
    def __init__(
        self,
        *,
        error: Optional["_models.ErrorDetail"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class Operation(_Model):
    """REST API Operation.

    :ivar name: The name of the operation, as per Resource-Based Access Control (RBAC). Examples:
     "Microsoft.Compute/virtualMachines/write", "Microsoft.Compute/virtualMachines/capture/action".
    :vartype name: str
    :ivar is_data_action: Whether the operation applies to data-plane. This is "true" for
     data-plane operations and "false" for Azure Resource Manager/control-plane operations.
    :vartype is_data_action: bool
    :ivar display: Localized display information for this particular operation.
    :vartype display: ~azure.mgmt.artifactsigning.models.OperationDisplay
    :ivar origin: The intended executor of the operation; as in Resource Based Access Control
     (RBAC) and audit logs UX. Default value is "user,system". Known values are: "user", "system",
     and "user,system".
    :vartype origin: str or ~azure.mgmt.artifactsigning.models.Origin
    :ivar action_type: Extensible enum. Indicates the action type. "Internal" refers to actions
     that are for internal only APIs. "Internal"
    :vartype action_type: str or ~azure.mgmt.artifactsigning.models.ActionType
    """

    name: Optional[str] = rest_field(visibility=["read"])
    """The name of the operation, as per Resource-Based Access Control (RBAC). Examples:
     \"Microsoft.Compute/virtualMachines/write\",
     \"Microsoft.Compute/virtualMachines/capture/action\"."""
    is_data_action: Optional[bool] = rest_field(name="isDataAction", visibility=["read"])
    """Whether the operation applies to data-plane. This is \"true\" for data-plane operations and
     \"false\" for Azure Resource Manager/control-plane operations."""
    display: Optional["_models.OperationDisplay"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Localized display information for this particular operation."""
    origin: Optional[Union[str, "_models.Origin"]] = rest_field(visibility=["read"])
    """The intended executor of the operation; as in Resource Based Access Control (RBAC) and audit
     logs UX. Default value is \"user,system\". Known values are: \"user\", \"system\", and
     \"user,system\"."""
    action_type: Optional[Union[str, "_models.ActionType"]] = rest_field(name="actionType", visibility=["read"])
    """Extensible enum. Indicates the action type. \"Internal\" refers to actions that are for
     internal only APIs. \"Internal\""""

    @overload
    def __init__(
        self,
        *,
        display: Optional["_models.OperationDisplay"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class OperationDisplay(_Model):
    """Localized display information for an operation.

    :ivar provider: The localized friendly form of the resource provider name, e.g. "Microsoft
     Monitoring Insights" or "Microsoft Compute".
    :vartype provider: str
    :ivar resource: The localized friendly name of the resource type related to this operation.
     E.g. "Virtual Machines" or "Job Schedule Collections".
    :vartype resource: str
    :ivar operation: The concise, localized friendly name for the operation; suitable for
     dropdowns. E.g. "Create or Update Virtual Machine", "Restart Virtual Machine".
    :vartype operation: str
    :ivar description: The short, localized friendly description of the operation; suitable for
     tool tips and detailed views.
    :vartype description: str
    """

    provider: Optional[str] = rest_field(visibility=["read"])
    """The localized friendly form of the resource provider name, e.g. \"Microsoft Monitoring
     Insights\" or \"Microsoft Compute\"."""
    resource: Optional[str] = rest_field(visibility=["read"])
    """The localized friendly name of the resource type related to this operation. E.g. \"Virtual
     Machines\" or \"Job Schedule Collections\"."""
    operation: Optional[str] = rest_field(visibility=["read"])
    """The concise, localized friendly name for the operation; suitable for dropdowns. E.g. \"Create
     or Update Virtual Machine\", \"Restart Virtual Machine\"."""
    description: Optional[str] = rest_field(visibility=["read"])
    """The short, localized friendly description of the operation; suitable for tool tips and detailed
     views."""


class Revocation(_Model):
    """Revocation details of the certificate.

    :ivar requested_at: The timestamp when the revocation is requested.
    :vartype requested_at: ~datetime.datetime
    :ivar effective_at: The timestamp when the revocation is effective.
    :vartype effective_at: ~datetime.datetime
    :ivar reason: Reason for revocation.
    :vartype reason: str
    :ivar remarks: Remarks for the revocation.
    :vartype remarks: str
    :ivar status: Status of the revocation. Known values are: "Succeeded", "InProgress", and
     "Failed".
    :vartype status: str or ~azure.mgmt.artifactsigning.models.RevocationStatus
    :ivar failure_reason: Reason for the revocation failure.
    :vartype failure_reason: str
    """

    requested_at: Optional[datetime.datetime] = rest_field(
        name="requestedAt", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The timestamp when the revocation is requested."""
    effective_at: Optional[datetime.datetime] = rest_field(
        name="effectiveAt", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The timestamp when the revocation is effective."""
    reason: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Reason for revocation."""
    remarks: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Remarks for the revocation."""
    status: Optional[Union[str, "_models.RevocationStatus"]] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Status of the revocation. Known values are: \"Succeeded\", \"InProgress\", and \"Failed\"."""
    failure_reason: Optional[str] = rest_field(
        name="failureReason", visibility=["read", "create", "update", "delete", "query"]
    )
    """Reason for the revocation failure."""

    @overload
    def __init__(
        self,
        *,
        requested_at: Optional[datetime.datetime] = None,
        effective_at: Optional[datetime.datetime] = None,
        reason: Optional[str] = None,
        remarks: Optional[str] = None,
        status: Optional[Union[str, "_models.RevocationStatus"]] = None,
        failure_reason: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class RevokeCertificate(_Model):
    """Defines the certificate revocation properties.

    :ivar serial_number: Serial number of the certificate. Required.
    :vartype serial_number: str
    :ivar thumbprint: Thumbprint of the certificate. Required.
    :vartype thumbprint: str
    :ivar effective_at: The timestamp when the revocation is effective. Required.
    :vartype effective_at: ~datetime.datetime
    :ivar reason: Reason for the revocation. Required.
    :vartype reason: str
    :ivar remarks: Remarks for the revocation.
    :vartype remarks: str
    """

    serial_number: str = rest_field(name="serialNumber", visibility=["read", "create", "update", "delete", "query"])
    """Serial number of the certificate. Required."""
    thumbprint: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Thumbprint of the certificate. Required."""
    effective_at: datetime.datetime = rest_field(
        name="effectiveAt", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The timestamp when the revocation is effective. Required."""
    reason: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Reason for the revocation. Required."""
    remarks: Optional[str] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Remarks for the revocation."""

    @overload
    def __init__(
        self,
        *,
        serial_number: str,
        thumbprint: str,
        effective_at: datetime.datetime,
        reason: str,
        remarks: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class SystemData(_Model):
    """Metadata pertaining to creation and last modification of the resource.

    :ivar created_by: The identity that created the resource.
    :vartype created_by: str
    :ivar created_by_type: The type of identity that created the resource. Known values are:
     "User", "Application", "ManagedIdentity", and "Key".
    :vartype created_by_type: str or ~azure.mgmt.artifactsigning.models.CreatedByType
    :ivar created_at: The timestamp of resource creation (UTC).
    :vartype created_at: ~datetime.datetime
    :ivar last_modified_by: The identity that last modified the resource.
    :vartype last_modified_by: str
    :ivar last_modified_by_type: The type of identity that last modified the resource. Known values
     are: "User", "Application", "ManagedIdentity", and "Key".
    :vartype last_modified_by_type: str or ~azure.mgmt.artifactsigning.models.CreatedByType
    :ivar last_modified_at: The timestamp of resource last modification (UTC).
    :vartype last_modified_at: ~datetime.datetime
    """

    created_by: Optional[str] = rest_field(name="createdBy", visibility=["read", "create", "update", "delete", "query"])
    """The identity that created the resource."""
    created_by_type: Optional[Union[str, "_models.CreatedByType"]] = rest_field(
        name="createdByType", visibility=["read", "create", "update", "delete", "query"]
    )
    """The type of identity that created the resource. Known values are: \"User\", \"Application\",
     \"ManagedIdentity\", and \"Key\"."""
    created_at: Optional[datetime.datetime] = rest_field(
        name="createdAt", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The timestamp of resource creation (UTC)."""
    last_modified_by: Optional[str] = rest_field(
        name="lastModifiedBy", visibility=["read", "create", "update", "delete", "query"]
    )
    """The identity that last modified the resource."""
    last_modified_by_type: Optional[Union[str, "_models.CreatedByType"]] = rest_field(
        name="lastModifiedByType", visibility=["read", "create", "update", "delete", "query"]
    )
    """The type of identity that last modified the resource. Known values are: \"User\",
     \"Application\", \"ManagedIdentity\", and \"Key\"."""
    last_modified_at: Optional[datetime.datetime] = rest_field(
        name="lastModifiedAt", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The timestamp of resource last modification (UTC)."""

    @overload
    def __init__(
        self,
        *,
        created_by: Optional[str] = None,
        created_by_type: Optional[Union[str, "_models.CreatedByType"]] = None,
        created_at: Optional[datetime.datetime] = None,
        last_modified_by: Optional[str] = None,
        last_modified_by_type: Optional[Union[str, "_models.CreatedByType"]] = None,
        last_modified_at: Optional[datetime.datetime] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from __future__ import annotations

import os
import sys
import datetime
import logging
import json
import pathlib
import glob
import subprocess
import csv
import calendar
from typing import Any

import requests
from github import Github, Auth

from ci_tools.parsing import ParsedSetup
from ci_tools.environment_exclusions import (
    IGNORE_PACKAGES,
    is_check_enabled,
    FILTER_EXCLUSIONS,
    IGNORE_FILTER,
)

logging.getLogger().setLevel(logging.INFO)

INACTIVE_CLASSIFIER = "Development Status :: 7 - Inactive"

IGNORE_FILTER.append("mgmt")
FILTER_EXCLUSIONS.append("azure-mgmt-core")
IGNORE_PACKAGES.extend([
    "azure-openai",
    "azure-storage-extensions",
    "azure-schemaregistry-avroserializer",
    "azure-eventhub-checkpointstoretable",
])

SDK_TEAM_OWNED_PACKAGES = [
    "azure-ai-documentintelligence",
    "azure-appconfiguration",
    "azure-containerregistry",
    "azure-core",
    "azure-mgmt-core",
    "azure-core-experimental",
    "azure-core-tracing-opencensus",
    "azure-core-tracing-openetelemetry",
    "azure-data-tables",
    "azure-eventgrid",
    "azure-eventhub",
    "azure-identity",
    "azure-keyvault-administration",
    "azure-keyvault-certificates",
    "azure-keyvault-keys",
    "azure-keyvault-secrets",
    "azure-monitor-ingestion",
    "azure-monitor-query",
    "azure-schemaregistry",
    "azure-search-documents",
    "azure-servicebus",
    "corehttp",
]


# We install each library from the dev feed. This installs mostly alpha versions.
# These libraries have dependencies on other libraries where an alpha version is not
# considered compatible with the given version specifier. Therefore, we add a second step
# to install/score these libraries separately.

# Format: "library_to_score": [dependencies_to_uninstall]
RESOLUTION_IMPOSSIBLE_LIBRARIES = {
    "azure-mixedreality-authentication": ["azure-mixedreality-remoterendering"],
    "azure-ai-ml": ["azure-storage-blob", "azure-storage-file-share", "azure-storage-file-datalake"],
    "azure-storage-blob-changefeed": ["azure-storage-blob"],
    "azure-storage-file-datalake": ["azure-storage-blob"],
    "azure-core": ["azure-core-experimental", "azure-core-tracing-opencensus", "azure-core-tracing-opentelemetry"],
    "azure-monitor-opentelemetry": ["azure-core-tracing-opentelemetry"],
    "azure-ai-evaluation": ["azure-monitor-opentelemetry-exporter", "azure-monitor-opentelemetry"],
    "azure-ai-language-conversations-authoring": ["azure-ai-language-conversations"],
    "azure-ai-textanalytics-authoring": ["azure-ai-textanalytics"],
}


def add_entity(package: str, packages_to_score: dict[str, Any], entities: list[dict[str, Any]]) -> None:
    d = packages_to_score[package]["Date"]
    entity = {
        "Package": package,
        "Date": d,
        "LatestVersion": packages_to_score[package]["LatestVersion"],
        "Score": packages_to_score[package]["Score"],
        "PyTyped": packages_to_score[package]["PyTyped"],
        "Pyright": packages_to_score[package]["Pyright"],
        "Mypy": packages_to_score[package]["Mypy"],
        "Samples": packages_to_score[package]["Samples"],
        "Verifytypes": packages_to_score[package]["Verifytypes"],
        "SDKTeamOwned": packages_to_score[package]["SDKTeamOwned"],
        "Active": packages_to_score[package]["Active"]
    }
    entities.append(entity)


def install(packages: list[str]) -> None:
    commands = [
        sys.executable,
        "-m",
        "pip",
        "install",
    ]

    commands.extend(packages)
    subprocess.check_call(commands)


def uninstall_deps(deps: list[str]) -> None:
    commands = [
        sys.executable,
        "-m",
        "pip",
        "uninstall",
        "-y"
    ]

    commands.extend(deps)
    subprocess.check_call(commands)


def score_package(package: str, packages_to_score: dict[str, Any], entities: list[dict[str, Any]]) -> None:
    try:
        logging.info(f"Running verifytypes on {package}")
        commands = [sys.executable, "-m", "pyright", "--verifytypes", packages_to_score[package]["Module"], "--ignoreexternal", "--outputjson"]
        response = subprocess.run(
            commands,
            check=True,
            capture_output=True,
        )
    except subprocess.CalledProcessError as e:
        if e.returncode != 1:
            logging.info(
                f"Running verifytypes for {package} failed: {e.stderr}"
            )
        else:
            report = json.loads(e.output)
    else:
        report = json.loads(response.stdout)  # package scores 100%
    pytyped_present = False if report["typeCompleteness"].get("pyTypedPath", None) is None else True
    packages_to_score[package].update({"PyTyped": pytyped_present})
    packages_to_score[package].update({"Score": round(report["typeCompleteness"]["completenessScore"] * 100, 1)})
    add_entity(package, packages_to_score, entities)


def get_alpha_installs(packages_to_score: dict[str, Any]) -> tuple[list[str], dict[str, list[str]]]:
    feed_id = "3572dbf9-b5ef-433b-9137-fc4d7768e7cc"
    feed_resp = requests.get(f"https://feeds.dev.azure.com/azure-sdk/public/_apis/packaging/feeds/{feed_id}/Packages?api-version=7.0")
    packages = json.loads(feed_resp.text)
    versions_to_install = {}

    # Dev feed doesn't default to latest version, so we have to check publish dates for most recent
    for package in packages["value"]:
        if package["name"] in packages_to_score:
            url = f"{package['url']}/Versions"
            versions = requests.get(url)
            version_list = json.loads(versions.text)["value"]
            latest_version = max(version_list, key=lambda v: datetime.datetime.strptime(v["publishDate"].split(".")[0], "%Y-%m-%dT%H:%M:%S"))
            versions_to_install[package["name"]] = latest_version["version"]

    extra_index_url = ("--extra-index-url", "https://pkgs.dev.azure.com/azure-sdk/public/_packaging/azure-sdk-for-python/pypi/simple", "--pre")
    first_round = []
    second_round = {}

    for package_name, version in versions_to_install.items():
        packages_to_score[package_name].update({'LatestVersion': version})
        package_info = [f"{package_name}=={version}", *extra_index_url]
        
        if package_name in RESOLUTION_IMPOSSIBLE_LIBRARIES:
            second_round[package_name] = package_info
        else:
            first_round.extend(package_info)

    return first_round, second_round


def is_package_inactive(package_path: str) -> bool:
    return INACTIVE_CLASSIFIER in ParsedSetup.from_path(package_path).classifiers


def skip_package(package_name: str) -> bool:
    return (
        (not package_name.startswith("azure") and package_name != "corehttp")
        or package_name in IGNORE_PACKAGES
        or package_name not in FILTER_EXCLUSIONS
        and any(identifier in package_name for identifier in IGNORE_FILTER)
    )


def get_packages_to_score() -> dict[str, dict[str, Any]]:
    dataplane = {}
    sdk_path = pathlib.Path(__file__).parent.parent.parent / "sdk"
    service_directories = glob.glob(f"{sdk_path}/*/", recursive=True)
    today = datetime.datetime.today()
    for service in service_directories:
        package_paths = glob.glob(f"{service}*/", recursive=True)
        for pkg_path in package_paths:
            package_path = pathlib.Path(pkg_path)
            package_name = package_path.name
            if skip_package(package_name) or not ((package_path / "setup.py").exists() or (package_path / "pyproject.toml").exists()):
                continue
            package_path = str(package_path)
            package_info = ParsedSetup.from_path(package_path)
            dataplane[package_name] = {"LatestVersion": package_info.version}
            dataplane[package_name].update({"Path": package_path})
            dataplane[package_name].update({"Module": package_info.namespace})
            dataplane[package_name].update({"Pyright": is_check_enabled(package_path, "pyright")})
            dataplane[package_name].update({"Mypy": is_check_enabled(package_path, "mypy")})
            dataplane[package_name].update({"Samples": is_check_enabled(package_path, "type_check_samples")})
            dataplane[package_name].update({"Verifytypes": is_check_enabled(package_path, "verifytypes")})
            dataplane[package_name].update({"Date": today})
            dataplane[package_name].update({"SDKTeamOwned": package_name in SDK_TEAM_OWNED_PACKAGES})
            dataplane[package_name].update({"Active": not is_package_inactive(package_path)})

    sorted_libs = {key: dataplane[key] for key in sorted(dataplane)}
    return sorted_libs


def append_results_to_csv(entities: list[dict[str, Any]]) -> None:
    typescore_path = pathlib.Path(__file__).parent

    github = Github(auth=Auth.Token(os.environ["GH_TOKEN"]))
    repo = github.get_repo("Azure/azure-sdk-for-python")

    path = "scripts/repo_type_completeness/typescores.csv"
    content = repo.get_contents(path, ref="python-sdk-type-completeness")
    decoded_content = content.decoded_content.decode("utf-8")

    with open(typescore_path / "typescores.csv", mode="a", newline='', encoding="utf-8") as file:
        file.write(decoded_content)
        writer = csv.writer(file)

        for data in entities:
            row = [
                data["Package"],
                data["Date"].strftime("%Y-%m-%dT%H:%M:%S.%fZ"),
                data["LatestVersion"],
                data["Score"],
                data["PyTyped"],
                data["Pyright"],
                data["Mypy"],
                data["Samples"],
                data["Verifytypes"],
                data["SDKTeamOwned"],
                data["Active"],
            ]
            writer.writerow(row)

    repo.update_file(
        path=path,
        message="Update typing dashboard",
        content=open(path, "rb").read(),
        branch="python-sdk-type-completeness",
        sha=content.sha
    )


def should_run() -> bool:
    """We only update the typing dashboard once a month on the Monday after release week.
    """
    pst = datetime.datetime.now(datetime.timezone(offset=datetime.timedelta(hours=-8)))
    today = pst.date()
    c = calendar.Calendar(firstweekday=calendar.SUNDAY)
    month_dates = c.monthdatescalendar(today.year, today.month)

    monday_after_release_week = None
    for week in month_dates:
        for day in week:
            if day.weekday() == calendar.FRIDAY and day.month == today.month:
                monday_after_release_week = day + datetime.timedelta(days=10)
                return today == monday_after_release_week

    return False


def update_main_typescores() -> None:
    if not should_run():
        logging.info(f"Skipping type scoring update - only runs once a month on the Monday after release week.")
        return

    packages_to_score = get_packages_to_score()

    first_round, second_round = get_alpha_installs(packages_to_score)

    install(first_round)

    entities = []
    for package, _ in packages_to_score.items():
        if package in RESOLUTION_IMPOSSIBLE_LIBRARIES:
            continue
        score_package(package, packages_to_score, entities)

    for package, deps in RESOLUTION_IMPOSSIBLE_LIBRARIES.items():
        uninstall_deps(deps)
        install(second_round[package])
        score_package(package, packages_to_score, entities)

    append_results_to_csv(entities)


if __name__ == "__main__":
    update_main_typescores()



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.azurestackhci import AzureStackHCIClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-azurestackhci
# USAGE
    python put_deployment_settings.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = AzureStackHCIClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.deployment_settings.begin_create_or_update(
        resource_group_name="test-rg",
        cluster_name="myCluster",
        resource={
            "properties": {
                "arcNodeResourceIds": [
                    "/subscriptions/fd3c3665-1729-4b7b-9a38-238e83b0f98b/resourceGroups/ArcInstance-rg/providers/Microsoft.HybridCompute/machines/Node-1",
                    "/subscriptions/fd3c3665-1729-4b7b-9a38-238e83b0f98b/resourceGroups/ArcInstance-rg/providers/Microsoft.HybridCompute/machines/Node-2",
                ],
                "deploymentConfiguration": {
                    "scaleUnits": [
                        {
                            "deploymentData": {
                                "adouPath": "OU=ms169,DC=ASZ1PLab8,DC=nttest,DC=microsoft,DC=com",
                                "cluster": {
                                    "azureServiceEndpoint": "core.windows.net",
                                    "cloudAccountName": "myasestoragacct",
                                    "name": "testHCICluster",
                                    "witnessPath": "Cloud",
                                    "witnessType": "Cloud",
                                },
                                "domainFqdn": "ASZ1PLab8.nttest.microsoft.com",
                                "hostNetwork": {
                                    "enableStorageAutoIp": False,
                                    "intents": [
                                        {
                                            "adapter": ["Port2"],
                                            "adapterPropertyOverrides": {
                                                "jumboPacket": "1514",
                                                "networkDirect": "Enabled",
                                                "networkDirectTechnology": "iWARP",
                                            },
                                            "name": "Compute_Management",
                                            "overrideAdapterProperty": False,
                                            "overrideQosPolicy": False,
                                            "overrideVirtualSwitchConfiguration": False,
                                            "qosPolicyOverrides": {
                                                "bandwidthPercentage_SMB": "50",
                                                "priorityValue8021Action_Cluster": "7",
                                                "priorityValue8021Action_SMB": "3",
                                            },
                                            "trafficType": ["Compute", "Management"],
                                            "virtualSwitchConfigurationOverrides": {
                                                "enableIov": "True",
                                                "loadBalancingAlgorithm": "HyperVPort",
                                            },
                                        }
                                    ],
                                    "storageConnectivitySwitchless": True,
                                    "storageNetworks": [
                                        {
                                            "name": "Storage1Network",
                                            "networkAdapterName": "Port3",
                                            "storageAdapterIPInfo": [
                                                {
                                                    "ipv4Address": "10.57.48.60",
                                                    "physicalNode": "string",
                                                    "subnetMask": "255.255.248.0",
                                                }
                                            ],
                                            "vlanId": "5",
                                        }
                                    ],
                                },
                                "infrastructureNetwork": [
                                    {
                                        "dnsServerConfig": "UseDnsServer",
                                        "dnsServers": ["10.57.50.90"],
                                        "gateway": "255.255.248.0",
                                        "ipPools": [{"endingAddress": "10.57.48.66", "startingAddress": "10.57.48.60"}],
                                        "subnetMask": "255.255.248.0",
                                    }
                                ],
                                "isManagementCluster": True,
                                "namingPrefix": "ms169",
                                "observability": {
                                    "episodicDataUpload": True,
                                    "euLocation": False,
                                    "streamingDataClient": True,
                                },
                                "optionalServices": {"customLocation": "customLocationName"},
                                "physicalNodes": [
                                    {"ipv4Address": "10.57.51.224", "name": "ms169host"},
                                    {"ipv4Address": "10.57.53.236", "name": "ms154host"},
                                ],
                                "sdnIntegration": {
                                    "networkController": {
                                        "macAddressPoolStart": "00-0D-3A-1B-C7-21",
                                        "macAddressPoolStop": "00-0D-3A-1B-C7-29",
                                        "networkVirtualizationEnabled": True,
                                    }
                                },
                                "secrets": [
                                    {
                                        "eceSecretName": "BMCAdminUserCred",
                                        "secretLocation": "https://sclusterkvnirhci35.vault.azure.net/secrets/cluster-34232342-BmcAdminUser-f5bcc1d9-23af-4ae9-aca1-041d0f593a63/9276354aabfc492fa9b2cdbefb54ae4b",
                                        "secretName": "cluster1-BmcAdminUser-f5bcc1d9-23af-4ae9-aca1-041d0f593a63",
                                    },
                                    {
                                        "eceSecretName": "AzureStackLCMUserCredential",
                                        "secretLocation": "https://sclusterkvnirhci35.vault.azure.net/secrets/cluster-34232342-AzureStackLCMUserCredential-f5bcc1d9-23af-4ae9-aca1-041d0f593a63/9276354aabfc492fa9b2cdbefb54ae4c",
                                        "secretName": "cluster2-AzureStackLCMUserCredential-f5bcc1d9-23af-4ae9-aca1-041d0f593a63",
                                    },
                                ],
                                "secretsLocation": "/subscriptions/db4e2fdb-6d80-4e6e-b7cd-xxxxxxx/resourceGroups/test-rg/providers/Microsoft.KeyVault/vaults/abcd123",
                                "securitySettings": {
                                    "bitlockerBootVolume": True,
                                    "bitlockerDataVolumes": True,
                                    "credentialGuardEnforced": False,
                                    "driftControlEnforced": True,
                                    "drtmProtection": True,
                                    "hvciProtection": True,
                                    "sideChannelMitigationEnforced": True,
                                    "smbClusterEncryption": False,
                                    "smbSigningEnforced": True,
                                    "wdacEnforced": True,
                                },
                                "storage": {"configurationMode": "Express"},
                            },
                            "sbePartnerInfo": {
                                "credentialList": [
                                    {
                                        "eceSecretName": "DownloadConnectorCred",
                                        "secretLocation": "https://sclusterkvnirhci35.vault.azure.net/secrets/cluster-34232342-DownloadConnectorCred-f5bcc1d9-23af-4ae9-aca1-041d0f593a63/9276354aabfc492fa9b2cdbefb54ae4b",
                                        "secretName": "cluster1-DownloadConnectorCred-f5bcc1d9-23af-4ae9-aca1-041d0f593a63",
                                    }
                                ],
                                "partnerProperties": [
                                    {"name": "EnableBMCIpV6", "value": "false"},
                                    {"name": "PhoneHomePort", "value": "1653"},
                                    {"name": "BMCSecurityState", "value": "HighSecurity"},
                                ],
                                "sbeDeploymentInfo": {
                                    "family": "Gen5",
                                    "publisher": "Contoso",
                                    "sbeManifestCreationDate": "2023-07-25T02:40:33Z",
                                    "sbeManifestSource": "default",
                                    "version": "4.0.2309.13",
                                },
                            },
                        }
                    ],
                    "version": "string",
                },
                "deploymentMode": "Deploy",
                "operationType": "ClusterProvisioning",
            }
        },
    ).result()
    print(response)


# x-ms-original-file: 2026-04-01-preview/PutDeploymentSettings.json
if __name__ == "__main__":
    main()



# coding=utf-8
# pylint: disable=useless-super-delegation

from typing import Any, Mapping, overload

from .._utils.model_base import Model as _Model, rest_field


class InvalidAuth(_Model):
    """InvalidAuth.

    :ivar error: Required.
    :vartype error: str
    """

    error: str = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """Required."""

    @overload
    def __init__(
        self,
        *,
        error: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import functools

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.core.credentials import AzureKeyCredential
from azure.ai.textanalytics import TextAnalysisClient
from azure.ai.textanalytics.models import (
    MultiLanguageTextInput,
    MultiLanguageInput,
    TextPiiEntitiesRecognitionInput,
    AnalyzeTextPiiResult,
    ConfidenceScoreThreshold,
    ConfidenceScoreThresholdOverride,
    PiiActionContent,
)

TextAnalysisPreparer = functools.partial(
    EnvironmentVariableLoader,
    "text_analysis",
    text_analysis_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    text_analysis_key="fake_key",
)


class TestTextAnalysis(AzureRecordedTestCase):
    def create_client(self, endpoint: str, key: str) -> TextAnalysisClient:
        return TextAnalysisClient(endpoint, AzureKeyCredential(key))


class TestTextAnalysisCase_NewPIIThresholds(TestTextAnalysis):
    @TextAnalysisPreparer()
    @recorded_by_proxy
    def test_recognize_pii_confidence_score(self, text_analysis_endpoint, text_analysis_key):
        client = self.create_client(text_analysis_endpoint, text_analysis_key)

        # Input documents
        docs = [
            MultiLanguageInput(
                id="1",
                text="My name is John Doe. My ssn is 222-45-6789. My email is john@example.com. John Doe is my name.",
                language="en",
            )
        ]
        text_input = MultiLanguageTextInput(multi_language_inputs=docs)

        # Confidence score overrides
        ssn_override = ConfidenceScoreThresholdOverride(value=0.9, entity="USSocialSecurityNumber")
        email_override = ConfidenceScoreThresholdOverride(value=0.9, entity="Email")
        confidence_threshold = ConfidenceScoreThreshold(default=0.3, overrides=[ssn_override, email_override])
        # Parameters
        parameters = PiiActionContent(
            pii_categories=["All"], confidence_score_threshold=confidence_threshold
        )

        body = TextPiiEntitiesRecognitionInput(text_input=text_input, action_content=parameters)

        # Sync call
        result = client.analyze_text(body=body)

        # Basic result shape checks
        assert result is not None
        assert isinstance(result, AnalyzeTextPiiResult)
        assert result.results is not None
        assert result.results.documents is not None

        doc = result.results.documents[0]
        redacted = doc.redacted_text

        # Person should be masked out in text; SSN & Email should remain (filtered out as entities)
        assert "John Doe" not in redacted
        assert doc.entities is not None
        assert len(doc.entities) > 0

        # Person is present
        assert any(e.category == "Person" for e in doc.entities), "Expected at least one Person entity"

        # Verify SSN / Email are NOT returned as entities
        bad_categories = {"USSocialSecurityNumber", "Email"}
        bad_types = {"USSocialSecurityNumber", "Email"}

        for e in doc.entities:
            assert e.category not in bad_categories
            assert e.type not in bad_types



# pylint: disable=too-many-lines
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

""" Custom Sync Polling Methods for Azure Batch Operations."""

import time
from typing import Any, Callable, Optional

from azure.core.exceptions import AzureError, ResourceNotFoundError
from azure.core.pipeline import PipelineResponse
from azure.core.polling import PollingMethod

from .. import models as _models


class DeleteJobPollingMethod(PollingMethod):
    """Polling method for job delete operation.

    This class is used to poll the status of a job deletion operation.
    It checks the status of the job until it is deleted or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        job_id: str,
        *,
        polling_interval: int = 5,
    ):
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._job_id = job_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            self.update_status()
            if not self.finished():
                # add a delay if not done
                time.sleep(self._polling_interval)

    def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            job = self._client.get_job(self._job_id)

            if job.state == _models.BatchJobState.DELETING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Job no longer exists, deletion is complete
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Some other error occurred, operation failed
            self._status = "Failed"
            self._finished = True


class DisableJobPollingMethod(PollingMethod):
    """Polling method for job disable operation.

    This class is used to poll the status of a job disable operation.
    It checks the status of the job until it is disabled or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        job_id: str,
        *,
        polling_interval: int = 5,
    ):
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._status = "InProgress"
        self._finished = False
        self._job_id = job_id
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            self.update_status()
            if not self.finished():
                # add a delay if not done
                time.sleep(self._polling_interval)

    def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            job = self._client.get_job(self._job_id)

            if job.state == _models.BatchJobState.DISABLING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # If job doesn't exist it could've been deleted while disabling
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class EnableJobPollingMethod(PollingMethod):
    """Polling method for job enable operation.

    This class is used to poll the status of a job enable operation.
    It checks the status of the job until it is enabled or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        job_id: str,
        *,
        polling_interval: int = 5,
    ):
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._job_id = job_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            self.update_status()
            if not self.finished():
                # add a delay if not done
                time.sleep(self._polling_interval)

    def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            job = self._client.get_job(self._job_id)

            if job.state == _models.BatchJobState.ENABLING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # If job doesn't exist it could've been deleted while enabling
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class DeleteJobSchedulePollingMethod(PollingMethod):
    """Polling method for job schedule delete operation.

    This class is used to poll the status of a job schedule deletion operation.
    It checks the status of the job schedule until it is deleted or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        job_schedule_id: str,
        *,
        polling_interval: int = 5,
    ):
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._job_schedule_id = job_schedule_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    def run(self) -> None:
        while not self.finished():
            self.update_status()
            if not self.finished():
                # add a delay if not done
                time.sleep(self._polling_interval)

    def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            job_schedule = self._client.get_job_schedule(self._job_schedule_id)

            if job_schedule.state == _models.BatchJobScheduleState.DELETING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Job schedule no longer exists, deletion is complete
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class DeletePoolPollingMethod(PollingMethod):
    """Polling method for pool delete operation.
    This class is used to poll the status of a pool deletion operation.
    It checks the status of the pool until it is deleted or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        polling_interval: int = 5,
    ):
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            self.update_status()
            if not self.finished():
                # add a delay if not done
                time.sleep(self._polling_interval)

    def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            pool = self._client.get_pool(self._pool_id)

            if pool.state == _models.BatchPoolState.DELETING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Pool no longer exists, deletion is complete
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class DeallocateNodePollingMethod(PollingMethod):
    """Polling method for node deallocate operation.

    This class is used to poll the status of a node deallocation operation.
    It checks the status of the node until it is deallocated or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        node_id: str,
        polling_interval: int = 5,
    ):
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._node_id = node_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            self.update_status()
            if not self.finished():
                # add a delay if not done
                time.sleep(self._polling_interval)

    def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            node = self._client.get_node(self._pool_id, self._node_id)

            # If node not in DEALLOCATING state then completed
            # DEALLOCATED is too quick of a state to check for success
            if node.state == _models.BatchNodeState.DEALLOCATING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Node no longer exists, might have been removed from pool
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class RebootNodePollingMethod(PollingMethod):
    """Polling method for node reboot operation.

    This class is used to poll the status of a node reboot operation.
    It checks the status of the node until it is rebooted or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        node_id: str,
        polling_interval: int = 5,
    ):
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._node_id = node_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            self.update_status()
            if not self.finished():
                # add a delay if not done
                time.sleep(self._polling_interval)

    def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            node = self._client.get_node(self._pool_id, self._node_id)

            if node.state == _models.BatchNodeState.REBOOTING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Node could be deleted while rebooting
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class ReimageNodePollingMethod(PollingMethod):
    """Polling method for node reimage operation.

    This class is used to poll the status of a node reimage operation.
    It checks the status of the node until it is reimaged or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        node_id: str,
        polling_interval: int = 5,
    ):
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._node_id = node_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            self.update_status()
            if not self.finished():
                # add a delay if not done
                time.sleep(self._polling_interval)

    def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            node = self._client.get_node(self._pool_id, self._node_id)

            if node.state == _models.BatchNodeState.REIMAGING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Node could be deleted while reimaging
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class RemoveNodePollingMethod(PollingMethod):
    """Polling method for node remove operation.

    This class is used to poll the status of a node removal operation.
    It checks the status of the node until it is removed or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        polling_interval: int = 5,
    ):
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            self.update_status()
            if not self.finished():
                # add a delay if not done
                time.sleep(self._polling_interval)

    def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            pool = self._client.get_pool(self._pool_id)

            # Node removal is complete when the pool allocation state is STEADY
            # This means the pool is no longer resizing/removing nodes
            if pool.allocation_state == _models.AllocationState.STEADY:
                self._status = "Succeeded"
                self._finished = True
            else:
                self._status = "InProgress"
                self._finished = False

        except ResourceNotFoundError:
            # Pool could be deleted while removing nodes
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class ResizePoolPollingMethod(PollingMethod):
    """Polling method for pool resize operation.

    This class is used to poll the status of a pool resize operation.
    It checks the status of the pool until it is resized or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        polling_interval: int = 5,
    ):
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            self.update_status()
            if not self.finished():
                # add a delay if not done
                time.sleep(self._polling_interval)

    def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            pool = self._client.get_pool(self._pool_id)

            # Pool resize is complete when the pool allocation state is STEADY
            # This means the pool is no longer resizing (adding/removing nodes)
            if pool.allocation_state == _models.AllocationState.STEADY:
                self._status = "Succeeded"
                self._finished = True
            else:
                self._status = "InProgress"
                self._finished = False

        except ResourceNotFoundError:
            # Pool could be deleted while resizing
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class StartNodePollingMethod(PollingMethod):
    """Polling method for node start operation.

    This class is used to poll the status of a node start operation.
    It checks the status of the node until it is started or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        node_id: str,
        polling_interval: int = 5,
    ):
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._node_id = node_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            self.update_status()
            if not self.finished():
                # add a delay if not done
                time.sleep(self._polling_interval)

    def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            node = self._client.get_node(self._pool_id, self._node_id)

            if node.state == _models.BatchNodeState.STARTING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Node could be deleted during starting
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class StopPoolResizePollingMethod(PollingMethod):
    """Polling method for pool stop resize operation.

    This class is used to poll the status of a pool stop resize operation.
    It checks the status of the pool until it is stopped or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        pool_id: str,
        *,
        polling_interval: int = 5,
    ):
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._pool_id = pool_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            self.update_status()
            if not self.finished():
                # add a delay if not done
                time.sleep(self._polling_interval)

    def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            pool = self._client.get_pool(self._pool_id)

            # Pool stop resize is complete when the pool allocation state is STEADY
            # This means the pool has stopped resizing and is stable
            if pool.allocation_state == _models.AllocationState.STEADY:
                self._status = "Succeeded"
                self._finished = True
            else:
                self._status = "InProgress"
                self._finished = False

        except ResourceNotFoundError:
            # Pool could be deleted during stop resize operation
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class TerminateJobPollingMethod(PollingMethod):
    """Polling method for job termination operation.

    This class is used to poll the status of a job termination operation.
    It checks the status of the job until it is terminated or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        job_id: str,
        *,
        polling_interval: int = 5,
    ):
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._job_id = job_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            self.update_status()
            if not self.finished():
                # add a delay if not done
                time.sleep(self._polling_interval)

    def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            job = self._client.get_job(self._job_id)

            if job.state == _models.BatchJobState.TERMINATING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Job could be deleted while terminating
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True


class TerminateJobSchedulePollingMethod(PollingMethod):
    """Polling method for job schedule termination operation.

    This class is used to poll the status of a job schedule termination operation.
    It checks the status of the job schedule until it is terminated or an error occurs.
    """

    def __init__(
        self,
        client: Any,
        initial_response: PipelineResponse,
        deserialization_callback: Optional[Callable],
        job_schedule_id: str,
        *,
        polling_interval: int = 5,
    ):
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._job_schedule_id = job_schedule_id
        self._status = "InProgress"
        self._finished = False
        self._polling_interval = polling_interval

    def initialize(
        self, client: Any, initial_response: PipelineResponse, deserialization_callback: Optional[Callable]
    ) -> None:
        pass

    def status(self) -> str:
        return self._status

    def finished(self) -> bool:
        return self._finished

    def resource(self) -> Optional[Any]:
        if self._deserialization_callback and self._finished:
            return self._deserialization_callback()
        return None

    def run(self) -> None:
        """The polling loop.

        The polling should call the status monitor, evaluate and set the current status,
        insert delay between polls, and continue polling until a terminal state is reached.
        """
        while not self.finished():
            self.update_status()
            if not self.finished():
                # add a delay if not done
                time.sleep(self._polling_interval)

    def update_status(self) -> None:
        """Update the current status of the LRO by calling the status monitor
        and then using the polling strategy's get_status() to set the status."""
        try:
            job_schedule = self._client.get_job_schedule(self._job_schedule_id)

            # Job schedule termination is complete when it's no longer in TERMINATING state
            if job_schedule.state == _models.BatchJobScheduleState.TERMINATING:
                self._status = "InProgress"
                self._finished = False
            else:
                self._status = "Succeeded"
                self._finished = True

        except ResourceNotFoundError:
            # Job schedule could be deleted while terminating
            self._status = "Succeeded"
            self._finished = True
        except AzureError:
            # Another error occurred so LRO failed
            self._status = "Failed"
            self._finished = True



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.billingbenefits import BillingBenefitsMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-billingbenefits
# USAGE
    python savings_plan_validate_purchase.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = BillingBenefitsMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.benefit.validate(
        body={
            "benefits": [
                {
                    "benefitType": "SavingsPlan",
                    "properties": {
                        "appliedScopeProperties": {
                            "resourceGroupId": "/subscriptions/10000000-0000-0000-0000-000000000000/resourceGroups/testrg"
                        },
                        "appliedScopeType": "Single",
                        "billingScopeId": "/subscriptions/10000000-0000-0000-0000-000000000000",
                        "commitment": {"amount": 15.23, "currencyCode": "USD", "grain": "Hourly"},
                        "displayName": "ComputeSavingsPlan_2021-07-01",
                        "term": "P1Y",
                    },
                    "sku": {"name": "Compute_Savings_Plan"},
                },
                {
                    "benefitType": "SavingsPlan",
                    "properties": {
                        "appliedScopeProperties": {
                            "resourceGroupId": "/subscriptions/10000000-0000-0000-0000-000000000000/resourceGroups/RG"
                        },
                        "appliedScopeType": "Single",
                        "billingScopeId": "/subscriptions/10000000-0000-0000-0000-000000000000",
                        "commitment": {"amount": 20, "currencyCode": "USD", "grain": "Hourly"},
                        "displayName": "ComputeSavingsPlan_2021-07-01",
                        "term": "P1Y",
                    },
                    "sku": {"name": "Compute_Savings_Plan"},
                },
            ]
        },
    )
    print(response)


# x-ms-original-file: 2025-12-01-preview/SavingsPlanValidatePurchase.json
if __name__ == "__main__":
    main()
